/*
    Create LOGINs
*/
USE [master]
GO

IF NOT EXISTS (SELECT * FROM sys.server_principals WHERE name = N'ApplicationSQLUser1')
    CREATE LOGIN [ApplicationSQLUser1] WITH PASSWORD=N'123456', DEFAULT_DATABASE=[master], DEFAULT_LANGUAGE=[us_english], CHECK_EXPIRATION=OFF, CHECK_POLICY=OFF
GO

IF NOT EXISTS (SELECT * FROM sys.server_principals WHERE name = N'ApplicationSQLUser2')
    CREATE LOGIN [ApplicationSQLUser2] WITH PASSWORD=N'123456', DEFAULT_DATABASE=[master], DEFAULT_LANGUAGE=[us_english], CHECK_EXPIRATION=OFF, CHECK_POLICY=OFF
GO

GRANT CONNECT SQL to ApplicationSQLUser1 WITH GRANT OPTION;
GO 

grant CONNECT SQL to ApplicationSQLUser2 AS ApplicationSQLUser1;
GO

IF( (SELECT @@MICROSOFTVERSION / 0x01000000) >= 11 )
BEGIN 
    EXEC sp_executesql N'CREATE SERVER ROLE [ServerRoleOwnedByUser1] AUTHORIZATION [ApplicationSQLUser1];';
END;
GO

/*
    Create DATABASES
*/
CREATE DATABASE [Db1];
GO

CREATE DATABASE [Db2];
GO


/* 
    Set MAPPINGS
    create schemas
    set schema ownership and default schemas
*/


USE [Db1]
CREATE USER [UserA] FOR LOGIN [ApplicationSQLUser1]
GO
CREATE USER [UserB] FOR LOGIN [ApplicationSQLUser2]
GO
CREATE ROLE [RoleOwnedByUserA] AUTHORIZATION [UserA];
GO
ALTER ROLE [db_ddladmin] ADD MEMBER [UserA]
GO
CREATE SCHEMA ApplicationSchema1 AUTHORIZATION UserA;
GO

ALTER USER [UserA] WITH DEFAULT_SCHEMA=[ApplicationSchema1]
GO

ALTER USER [UserB] WITH DEFAULT_SCHEMA=[ApplicationSchema1]
GO

USE master;
GO
ALTER AUTHORIZATION ON DATABASE::Db2 TO ApplicationSQLUser1;
/*
    maps ApplicationSQLUser1 to [dbo] database user in Db2 database
*/
GO


/*
    Create stored procedure referencing UserA in Db1
*/  

Use Db1;
GO
CREATE PROCEDURE [ApplicationSchema1].[sp_ExecuteAsUserA]
WITH EXECUTE AS 'UserA'
AS
BEGIN
	SELECT @@SERVERNAME 
END;
GO


grant execute on OBJECT::[ApplicationSchema1].[sp_ExecuteAsUserA] to UserB AS UserA;
GO

/* ======================================================================================================================
*/

RAISERROR('Attempt to run the whole script',12,1); 
RETURN;

/* ======================================================================================================================
    Drop UserA in Db1
*/

USE Db1 ;
GO

grant execute on OBJECT::[ApplicationSchema1].[sp_ExecuteAsUserA] to UserB AS UserA;
GO
ALTER AUTHORIZATION ON SCHEMA::ApplicationSchema1 TO [dbo] ;
GO

ALTER AUTHORIZATION ON ROLE::RoleOwnedByUserA TO [dbo] ;
GO

ALTER PROCEDURE [ApplicationSchema1].[sp_ExecuteAsUserA]
WITH EXECUTE AS 'dbo'
AS
BEGIN
    SELECT @@SERVERNAME 
END;
GO 

DROP USER UserA ;
GO



/* ======================================================================================================================
*/

RAISERROR('Attempt to run the whole script',12,1); 
RETURN;

/* ======================================================================================================================
    Drop ApplicationSQLUser1
*/

USE master ;
GO
REVOKE CONNECT SQL FROM ApplicationSQLUser2  AS ApplicationSQLUser1;
GO
GRANT CONNECT SQL TO ApplicationSQLUser2 AS SA;
GO
ALTER AUTHORIZATION ON SERVER ROLE::[ServerRoleOwnedByUser1] TO [sa] ;
GO
ALTER AUTHORIZATION ON DATABASE::Db2 TO sa;
GO
DROP LOGIN [ApplicationSQLUser1];
GO



/* ======================================================================================================================
*/

RAISERROR('Attempt to run the whole script',12,1); 
RETURN;

/* ======================================================================================================================
    Cleanups
*/

USE master;
GO
DROP DATABASE [Db1];
GO
DROP DATABASE [Db2];
GO
/*
Already dropped (this is the purpose of this article)
DROP LOGIN [ApplicationSQLUser1];
GO
*/

DROP LOGIN [ApplicationSQLUser2];
GO

IF( (SELECT @@MICROSOFTVERSION / 0x01000000) >= 11 )
BEGIN 
    EXEC sp_executesql N'DROP SERVER ROLE [ServerRoleOwnedByUser1];';
END;
GO
